
# Adam's .switch_shell
#
# Try switch shell if we're interactive, aiming for safety, but
# not so much that we end up hogging memory.
#
# $Id: .switch_shell,v 1.9 2001/09/07 18:34:01 adam Exp $
#
# Usage:
#
# . /path/to/.switch_shell [-d] [ /path/to/new_shell [ <new shell options> ]
#    -d turns on debugging

# Originally found here:
# http://www.zsh.org/mla/workers/2001/msg02421.html

# Heavily modified by:
# Juhapekka Tolvanen
# http://iki.fi/juhtolv

#run_shell=`$myshell -c "echo exec $myshell" || echo :`
#    eval $run_shell '"$@"'

#run_shell=`$myshell -f -c "echo exec $myshell" || echo :`
#  eval $run_shell '"$@"'

#eval `$myshell -c "echo exec $myshell" || echo :` '"$@"'

######################################################################
# You can configure amount of paranoia and carefulness here:

# Uncomment this if switching dangerously (with exec) works generally well:
DANGEROUS_EXEC="1"
# Uncomment this, if binaries of better shells and their libraries
# are never b0rken:
NEVER_BROKEN="1"
# Uncomment this, if resize-command is always available:
#DANGEROUS_RESIZE="1"

######################################################################

#echo "~/.switch_shell"

#HOME_SWITCH_SHELL="1"
#export HOME_SWITCH_SHELL

if [ "$1" = '-d' ]; then
    debug=yes
    shift
fi

if [ -n "$1" ]; then
    myshell="$1"
    shift
else
    # Sensible default shell to switch to.
    myshell="/bin/zsh"
fi

myshell_args="$@"

switch_shell_safely () {
    # we do this rather than exec() just in case $myshell fails to run.
    [ -n "$debug" ] && echo "Switching to $myshell safely ..."
    NO_SWITCH="yes"
    export NO_SWITCH
    SHELL="$myshell"
    export SHELL
    unset HOME_SHINIT HOME_BASHRC HOME_BASH_COMMON
    $myshell $myshell_args "$@" && exit
}

switch_shell_dangerously () {
    [ -n "$debug" ] && echo "Switching to $myshell dangerously ..."
    NO_SWITCH="yes"
    export NO_SWITCH
    SHELL="$myshell"
    export SHELL
    unset HOME_SHINIT HOME_BASHRC HOME_BASH_COMMON
    exec $myshell $myshell_args "$@"
}

switch_shell () {
    if [ -n "$NO_SWITCH" ]; then
        [ -n "$debug" ] && echo 'Shell switching disabled by $NO_SWITCH; Aborting switch.'
        return
    fi

    if ! tty > /dev/null 2>&1
    then
        [ -n "$debug" ] && echo 'This is not a tty. Aborting switch'
        return
    fi


    if type resize > /dev/null 2>&1
    then
        eval `resize -u 2> /dev/null` 2> /dev/null
        if test ! -n "$COLUMNS"
        then
            [ -n "$debug" ] && echo '$COLUMNS does not exist. Aborting switch'
            return
        elif test ! -n "$LINES"
        then
            [ -n "$debug" ] && echo '$LINES does not exist. Aborting switch'
            return
        fi
    else
        [ -n "$debug" ] && echo 'resize -command does not exist. Install it or ask sys admin to install it.'
        if test ! -n "$COLUMNS"
        then
            [ -n "$debug" ] && echo '$COLUMNS does not exist.'
            if test "$DANGEROUS_RESIZE" = "1"
            then
                [ -n "$debug" ] && echo 'Aborting switch.'
                return
            fi
        elif test ! -n "$LINES"
        then
            [ -n "$debug" ] && echo '$LINES does not exist.'
            if test "$DANGEROUS_RESIZE" = "1"
            then
                [ -n "$debug" ] && echo 'Aborting switch.'
                return
            fi
        fi
    fi

    if test ! -n "$TERM"
    then
        [ -n "$debug" ] && echo '$TERM does not exist. Aborting switch'
        return
    fi

    if [ ! -x $myshell ]; then
        [ -n "$debug" ] && echo "$myshell not executable; Aborting switch."
        return
    fi

    if test $NEVER_BROKEN="1"
    then
        :
    else
        if ! $myshell -c "echo """
        then
            [ -n "$debug" ] && echo 'Shell is somehow b0rken. Aborting switch.'
            return
        fi
    fi

    if test $DANGEROUS_EXEC="1"
    then
        switch_shell_dangerously "$@"
    else
        if test -n "$SHLVL"
        then
            case "$SHLVL" in
                1) # login shell, be careful
                [ -n "$debug" ] && echo 'SHLVL = 1'
                switch_shell_safely "$@"
                ;;
                *) # other shell, be risky and save memory
                [ -n "$debug" ] && echo 'SHLVL != 1'
                switch_shell_dangerously "$@"
            esac
        else
            case "$-" in
                *l*) # login shell, be careful
                [ -n "$debug" ] && echo '$- incl. l'
                switch_shell_safely "$@"
                ;;
                *) # other shell, be risky and save memory
                [ -n "$debug" ] && echo '$- does not incl. l'
                switch_shell_dangerously "$@"
            esac
        fi
    fi
}

# only switch if we're interactive
case "$-" in
    *i*)
    switch_shell "$@"
    ;;
    *)
    :
esac

